@@ -39,7 +39,9 @@
 #include "atexit.h"
 
 /* Make this a weak reference to avoid pulling in malloc.  */
+#ifndef MALLOC_PROVIDED
 void * malloc(size_t) _ATTRIBUTE((__weak__));
+#endif
 
 #ifdef _LITE_EXIT
 /* As __call_exitprocs is weak reference in lite exit, make a
@@ -48,7 +50,7 @@ const void * __atexit_dummy = &__call_exitprocs;
 #endif
 
 #ifndef __SINGLE_THREAD__
-extern _LOCK_RECURSIVE_T __atexit_lock;
+extern _LOCK_RECURSIVE_T __atexit_recursive_mutex;
 #endif
 
 #ifdef _REENT_GLOBAL_ATEXIT
@@ -63,18 +65,16 @@ static struct _atexit _global_atexit0 = _ATEXIT_INIT;
  */
 
 int
-_DEFUN (__register_exitproc,
-	(type, fn, arg, d),
-	int type _AND
-	void (*fn) (void) _AND
-	void *arg _AND
+__register_exitproc (int type,
+	void (*fn) (void),
+	void *arg,
 	void *d)
 {
   struct _on_exit_args * args;
   register struct _atexit *p;
 
 #ifndef __SINGLE_THREAD__
-  __lock_acquire_recursive(__atexit_lock);
+  __lock_acquire_recursive(__atexit_recursive_mutex);
 #endif
 
   p = _GLOBAL_ATEXIT;
@@ -91,7 +91,7 @@ _DEFUN (__register_exitproc,
     {
 #ifndef _ATEXIT_DYNAMIC_ALLOC
 #ifndef __SINGLE_THREAD__
-      __lock_release_recursive(__atexit_lock);
+      __lock_release_recursive(__atexit_recursive_mutex);
 #endif
       return -1;
 #else
@@ -100,7 +100,7 @@ _DEFUN (__register_exitproc,
       if (!malloc)
 	{
 #ifndef __SINGLE_THREAD__
-	  __lock_release_recursive(__atexit_lock);
+	  __lock_release_recursive(__atexit_recursive_mutex);
 #endif
 	  return -1;
 	}
@@ -109,7 +109,7 @@ _DEFUN (__register_exitproc,
       if (p == NULL)
 	{
 #ifndef __SINGLE_THREAD__
-	  __lock_release_recursive(__atexit_lock);
+	  __lock_release_recursive(__atexit_recursive_mutex);
 #endif
 	  return -1;
 	}
@@ -133,7 +133,7 @@ _DEFUN (__register_exitproc,
 	{
 #ifndef _ATEXIT_DYNAMIC_ALLOC
 #ifndef __SINGLE_THREAD__
-	  __lock_release_recursive(__atexit_lock);
+	  __lock_release_recursive(__atexit_recursive_mutex);
 #endif
 	  return -1;
 #else
@@ -143,7 +143,7 @@ _DEFUN (__register_exitproc,
 	  if (args == NULL)
 	    {
 #ifndef __SINGLE_THREAD__
-	      __lock_release(__atexit_lock);
+	      __lock_release(__atexit_recursive_mutex);
 #endif
 	      return -1;
 	    }
@@ -163,7 +163,7 @@ _DEFUN (__register_exitproc,
     }
   p->_fns[p->_ind++] = fn;
 #ifndef __SINGLE_THREAD__
-  __lock_release_recursive(__atexit_lock);
+  __lock_release_recursive(__atexit_recursive_mutex);
 #endif
   return 0;
 }
